/*
 *  Copyright (C) 2002-23 - ntop.org
 *
 *  http://www.ntop.org/
 *
 * This code is proprietary code subject to the terms and conditions
 * defined in LICENSE file which is part of this source code package.
 */

#define MAX_PACKET_LEN              256

/* IMPORTANT Make sure you update ver9_templates when changing the value below */
#define MAX_NUM_CUSTOM_FLOW_FIELDS    300

#define MAX_CUSTOM_VALUE_LEN          128 /* bytes */
#define ZMQ_FLOW_RELAY_TOPIC          "flow_collection"

typedef enum {
  /*
    NOTE

    whenever this datastructure is updated
    you ought to also update
    dumpformat2ascii and printMetadata (plugin.c)
    dumpformat2enum (template.c)
    and elementDumpFormat2nDB (template.c)
  */
  dump_as_uint = 0, /* 1234567890 */
  dump_as_formatted_uint, /* 123'456 */
  dump_as_ip_port,
  dump_as_ip_proto,
  dump_as_ipv4_address,
  dump_as_ipv6_address,
  dump_as_mac_address,
  dump_as_epoch,
  dump_as_bool,
  dump_as_tcp_flags,
  dump_as_hex,
  dump_as_ascii,
  dump_as_float
} ElementDumpFormat;

typedef struct flow_ver9_template_field {
  u_int16_t fieldId;
  u_int16_t fieldLen;
  u_int8_t  isPenField;
  u_int32_t enterpriseId;
  u_int16_t equivalentStandardElementId;
} V9V10TemplateField;

struct flow_ver9_ipfix_template_elementids;

typedef struct customNetFlowField {
  struct flow_ver9_template_field template_field;
  u_int8_t len;
  u_char value[MAX_CUSTOM_VALUE_LEN];
  struct flow_ver9_ipfix_template_elementids *ie;
} customFields;

typedef struct {
  u_int32_t dst_as, dst_peer_as; /* dst peer/origin Autonomous System */
  u_int32_t src_as, src_peer_as; /* source peer/origin Autonomous System */  
} as_info;
  
struct generic_netflow_record {
  /* v5 */
  IpAddress srcaddr;    /* Source IP Address */
  IpAddress dstaddr;    /* Destination IP Address */
  IpAddress nexthop;    /* Next hop router's IP Address */
  u_int32_t input;      /* Input interface index */
  u_int32_t output;     /* Output interface index */
  u_int64_t sentPkts, rcvdPkts;
  u_int64_t sentOctets, rcvdOctets;
  u_int32_t first;      /* SysUptime at start of flow */
  u_int32_t last;       /* and of last packet of the flow */
  u_int16_t srcport;    /* TCP/UDP source port number (.e.g, FTP, Telnet, etc.,or equivalent) */
  u_int16_t dstport;    /* TCP/UDP destination port number (.e.g, FTP, Telnet, etc.,or equivalent) */
  u_int8_t  tcp_flags;  /* Cumulative OR of tcp flags */
  u_int8_t  proto;      /* IP protocol, e.g., 6=TCP, 17=UDP, etc... */
  u_int8_t  tos;        /* IP Type-of-Service */
  u_int8_t  minTTL, maxTTL; /* IP Time-to-Live */
  as_info   as;         /* ASN info */
  u_int8_t  dst_mask;   /* destination route's mask bits */
  u_int8_t  src_mask;   /* source route's mask bits */

  /* v9 */
  IpAddress bgpNextHop;    /* Next hop router's IP Address */
  u_int16_t vlanId_src, vlanId_dst, icmpTypeCode;
  u_int8_t icmpType, icmpCode;
  u_int8_t firewallEvent;
  u_int8_t flowEndReason;

  /*
    Collected info: if 0 it means they have not been
    set so we use the nprobe default (-E)
  */
  u_int8_t engine_type, engine_id;
  u_int32_t senderIP4Address;

  /* IPFIX */
  u_int32_t firstEpoch /* nw byte order*/, lastEpoch /* nw byte order*/, flowDuration /* host byte order */;
  u_int8_t src_mac[6], dst_mac[6];

  struct {
    u_int32_t privateEnterpriseNumber;
    u_int16_t ndpi_proto;

    u_int16_t numSrcFragments, numDstFragments;
    u_int32_t clientNwLatency, serverNwLatency, applLatency;
    u_int32_t retransmittedInPkts, retransmittedOutPkts;
    u_int32_t retransmittedInBytes, retransmittedOutBytes;
    u_int32_t oooInPkts, oooOutPkts;

    /* VoIP Extensions */
    char sip_call_id[50], sip_calling_party[50], sip_called_party[50];
  } ntop;

  struct {
    u_int8_t hasSampling;
    u_int16_t packet_len /* 103 */, original_packet_len /* 312/242 */, packet_offset /* 102 */;
    u_int32_t samplingPopulation /* 310 */, observationPointId /* 300 */;
    u_int16_t selectorId /* 302 */;
    u_char packet[MAX_PACKET_LEN] /* 104 */;

    /* Cisco NBAR 2 */
    u_int32_t nbar2_application_id /* NBAR - 95 */;
  } cisco;

  struct {
    u_int64_t imsi;
    u_int32_t src_addr;
  } nokia;
    
  IpAddress natSrcAddr;
  IpAddress natDstAddr;
  u_int32_t ingressVRFID, egressVRFID;
  u_int16_t natSrcTransportPort, natDstTransportPort, natPortRangeStart, natPortRangeEnd;
  u_int8_t natRealm;
  u_int8_t natEvent;
  u_int8_t flow_direction;
  u_int8_t forwarding_status;
  char     interface_name[48], reverse_interface_name[48];

  /* Option template: application types */
  u_int8_t selector_id;
  u_int32_t application_id;
  char application_name[25];

  /* sFlow-like packet analysis */
  u_int16_t frameLen, frameCaplen;
  u_char frameSection[1514];
  
  /* Dynamic field collection */
  u_int16_t numCustomFields;
  struct customNetFlowField customFields[MAX_NUM_CUSTOM_FLOW_FIELDS+1];

  struct {
    char ssid[33 /* 32 + \0 */];
    u_int8_t access_point_mac[6];
  } wifi;
};

extern void setCollectionFilter(char *filter);
extern void freeCollectionFilters(void);
extern void loadInformationElementsMapper(void);
extern struct flow_ver9_ipfix_template_elementids* findInformationElement(u_int32_t id,
									  u_int16_t enterpriseId);
extern void loadCollectionReforge(char *path);
extern u_int getMaxNumLicenseCollectors();
extern u_int getMaxNumLicenseZMQExporters();
extern FlowHashBucket* handleGenericFlow(u_int32_t netflow_device_ip,
					 struct in6_addr *flow_sender_ip6,
					 u_int8_t flow_version,
					 u_int32_t recordActTime, u_int32_t recordSysUpTime,
					 struct generic_netflow_record *record);
extern void update_collection_ststs(u_int32_t probe_ipv4_address, u_int32_t source_id,
				    bool is_netflow_ipfix,
				    u_int32_t flow_sequence, u_int32_t num_flows);
extern u_int8_t collectionLicenseExceeded(u_int32_t fromHostV4, struct ndpi_in6_addr *fromHostV6);

/* timeseries.c */
extern void closeTimeseriesLog();
extern void checkCloseTimeseriesLog(u_int32_t when);
extern void dumpFlowTimeseries(struct generic_netflow_record *record);
extern void dumpsTimeseries(char *str);
